import React from 'react';
import styles from './styles.less';
import {renderTxt} from '@/utils/hoverPopover';
import {Popover} from "antd";

function hex2rgb(sColor) {
    var sColorChange = [];
    for (let i = 0; i < 5; i += 2) {
        sColorChange.push(parseInt("0x" + sColor.substr(i, 2)))
    }
    sColorChange.push(0.5);
    return "rgba(" + sColorChange.join(",") + ")";
}

/*
*@params value {value:any,text:string}[]
*@params options {color:string,id:string}[]
*@params maxTagCount number  最多显示的tag数量
*@params maxTagTextLength    最大显示的tag文本长度
*@params defaultColor        默认背静色

*/
export default function TagList(props) {
    const value = props.value || [];
    // const value = [{ value: 1, text: 'shfkshdfsdhgfsd' }, { value: 2, text: 'hsafhhskdgfhdskghdlg' }, { value: 3, text: 'sdfghsdlgdshjkghsdkghflsdgf' }]
    if (!Array.isArray(value) || value.length === 0) return null;
    const maxTagCount = props.maxTagCount || 2;
    const maxTagTextLength = props.maxTagTextLength || 8;
    let newArr = value.slice(0, maxTagCount).filter(item => item.value);
    return (
        value.length > newArr.length ? <Popover
            content={
                <div className={styles.popContainer}>
                    {value.map((item, index) => {
                        let color = '';
                        if (props.options && Array.isArray(props.options) && props.options.length > 0) {
                            const obj = props.options.find(ls => ls.id === item.value) || {};
                            color = obj.color || '';
                        }
                        return (
                            <div className={styles.item}
                                 style={{background: `#${(color || props.defaultColor || 'E9EEF9')}`}}
                                 key={item.value + index}>
                                <span className={styles.itemBgTxt}>{item.text}</span>
                            </div>
                        )
                    })}
                </div>
            }
        >
            <div className={styles.tagList}>
                {newArr.map((item, index) => {
                    let color = '';
                    let str = maxTagTextLength < item.text.length ? `${item.text.slice(0, maxTagTextLength)}...` : item.text;
                    if (props.options && Array.isArray(props.options) && props.options.length > 0) {
                        const obj = props.options.find(ls => ls.id === item.value) || {};
                        color = obj.color || '';
                    }
                    // let background=hex2rgb(color || props.defaultColor || 'E9EEF9');
                    return (
                        <div className={styles.item}
                             style={{background: `#${(color || props.defaultColor || 'E9EEF9')}`}}
                             key={item.value + index}>
                            {/*<span className={styles.itemBg} style={{ background: `#${color || props.defaultColor || 'E9EEF9'}` }}></span>*/}
                            <span className={styles.itemBgTxt}>{renderTxt(item.text, maxTagTextLength)}</span>
                        </div>
                    )
                })}
                {value.length > newArr.length ?
                    <span className={styles.moreBtn}>+{value.length - newArr.length}</span> : null}
            </div>
        </Popover> : <div className={styles.tagList}>
            {newArr.map((item, index) => {
                let color = '';
                let str = maxTagTextLength < item.text.length ? `${item.text.slice(0, maxTagTextLength)}...` : item.text;
                if (props.options && Array.isArray(props.options) && props.options.length > 0) {
                    const obj = props.options.find(ls => ls.id === item.value) || {};
                    color = obj.color || '';
                }
                // let background=hex2rgb(color || props.defaultColor || 'E9EEF9');
                return (
                    <div className={styles.item}
                         style={{background: `#${(color || props.defaultColor || 'E9EEF9')}`}}
                         key={item.value + index}>
                        {/*<span className={styles.itemBg} style={{ background: `#${color || props.defaultColor || 'E9EEF9'}` }}></span>*/}
                        <span className={styles.itemBgTxt}>{renderTxt(item.text, maxTagTextLength)}</span>
                    </div>
                )
            })}
            {value.length > newArr.length ?
                <span className={styles.moreBtn}>+{value.length - newArr.length}</span> : null}
        </div>
    )
}
